import unittest

from easylib.io import FileMerger


class FileMergerTest(unittest.TestCase):

    def testFileMerger(self):
        m = FileMerger()
        input_files = ['easylib/io/file_reader.py', 'easylib/io/file_spliter.py']
        output_file = 'testdata/merged.txt'
        m.merge_files(input_files, output_file)


if __name__ == '__main__':
    unittest.main()



import unittest
import tensorflow as tf

from easylib.dl import KerasModelDatasetRunner

(train_images, train_labels), (test_images, test_labels) = tf.keras.datasets.mnist.load_data()

train_labels = train_labels[:1000]
test_labels = test_labels[:1000]

train_images = train_images[:1000].reshape(-1, 28 * 28) / 255.0
test_images = test_images[:1000].reshape(-1, 28 * 28) / 255.0

print(type(train_images))
print(train_images.shape)


class DatasetRunnerTest(unittest.TestCase):

    def createModel(self):
        model = tf.keras.models.Sequential([
            tf.keras.layers.Dense(512, activation='relu', input_shape=(784,)),
            tf.keras.layers.Dropout(0.2),
            tf.keras.layers.Dense(10, activation='softmax')
        ])

        model.compile(optimizer='adam',
                      loss='sparse_categorical_crossentropy',
                      metrics=['accuracy'])

        return model

    def testKerasModelDatasetRunner(self):
        model = self.createModel()
        r = KerasModelDatasetRunner(model, model_dir='/tmp/mnist', model_name='mnist')

        image = tf.data.Dataset.from_tensor_slices(train_images)
        label = tf.data.Dataset.from_tensor_slices(train_labels)
        dataset = tf.data.Dataset.zip((image, label)).shuffle(10000).batch(32)
        r.configs['epochs'] = 30
        r.train(dataset, ckpt='/tmp/mnist/mnist-0015.ckpt')


if __name__ == '__main__':
    unittest.main()



import logging
import os


class FileMerger:

    def __init__(self,
                 read_mode='rt',
                 write_mode='wt',
                 read_encoding='utf8',
                 write_encoding='utf8',
                 read_buffering=8192,
                 write_buffering=8192):
        self.read_mode = read_mode
        self.write_mode = write_mode
        self.read_encoding = read_encoding
        self.write_encoding = write_encoding
        self.read_buffering = read_buffering
        self.write_buffering = write_buffering

    def merge_files(self, input_files, output_file):
        path, name = os.path.split(output_file)
        if not os.path.exists(path):
            os.makedirs(path)
        with open(output_file, mode=self.write_mode, encoding=self.write_encoding,
                  buffering=self.write_buffering) as fout:
            for f in input_files:
                if not os.path.exists(f):
                    logging.info('File does not exist: %s' % f)
                    continue
                with open(f, mode=self.read_mode, encoding=self.read_encoding, buffering=self.read_buffering) as fin:
                    for line in fin:
                        fout.write(line)
                logging.info('Finished write %s to %s' % (f, output_file))
        logging.info('Done!')



import logging
import os

import tensorflow as tf


class DatasetRunner:
    """Tensorflow 2.x models runner, using tf.data API."""

    def __init__(self, model: tf.keras.Model, model_dir, model_name='model'):
        """Init.

        Args:
            model: Model, instance of `tf.keras.Model`
            model_dir: The directory to save model files
            model_name: The name of the model, used as checkpoint file names.
        """
        self.model = model
        self.model_dir = model_dir
        self.model_name = model_name

    def train(self, dataset, val_dataset=None, ckpt=None):
        """Train the model.

        Args:
            dataset: Dataset for training, instance of `tf.data.Dataset`
            val_dataset: Dataset for validation, instance of `tf.data.Dataset`
            ckpt: The checkpoint dir or path
        """
        raise NotImplementedError()

    def eval(self, dataset, ckpt=None):
        """Evaluate the model.

        Args:
            dataset: Dataset for evaluation, instance of `tf.data.Dataset`
            ckpt: The checkpoint dir or path
        """
        raise NotImplementedError()

    def predict(self, dataset, ckpt=None):
        """Predict the model.

        Args:
            dataset: Dataset for prediction
            ckpt: The checkpoint dir or path
        """
        raise NotImplementedError()

    def export(self, path, ckpt=None):
        """Export the saved model.

        Args:
            path: The directory to save the exported model
            ckpt: The checkpoint dir or path
        """
        self._load_weights(ckpt)
        tf.keras.experimental.export_saved_model(self.model, path)

    def _load_weights(self, ckpt=None):
        if not ckpt:
            ckpt = self.model_dir
        if os.path.isdir(ckpt):
            # empty dir
            if len(os.listdir(ckpt)) == 0:
                return
            if all('.ckpt' not in f for f in os.listdir(ckpt)):
                return
            ckpt = tf.train.latest_checkpoint(ckpt)
        self.model.load_weights(ckpt)
        logging.info('Load weights from ckpt: %s' % ckpt)


class KerasModelDatasetRunner(DatasetRunner):
    """Train, eval, predict and export tf.keras.Model, using `tf.data` API."""

    def __init__(self,
                 model,
                 model_dir,
                 model_name='model',
                 train_callbacks=None,
                 eval_callbacks=None,
                 predict_callbacks=None,
                 configs=None):
        """Init.

        Args:
            model: Model, instance of `tf.keras.Model`
            model_dir: The directory to save model files
            model_name: The name of the model, used as checkpoint file names.
            train_callbacks: A list of extra callbacks for training
            eval_callbacks: A list of extra callbacks for evaluation
            predict_callbacks: A list of extra callbacks for prediction
            configs: A dict of configuration, see `default_configs()`
        """
        super(KerasModelDatasetRunner, self).__init__(model, model_dir, model_name)
        default_configs = self.default_config()
        if configs:
            default_configs.update(configs)
        self.configs = default_configs

        self.train_callbacks = None
        self.eval_callbacks = None
        self.predict_callbacks = None
        self._init_callbacks(train_callbacks, eval_callbacks, predict_callbacks)

    def train(self, dataset, val_dataset=None, ckpt=None):
        """Train the model.

        Args:
            dataset: Dataset for training, instance of `tf.data.Dataset`
            val_dataset: Dataset for validation, instance of `tf.data.Dataset`
            ckpt: The checkpoint dir or path

        Returns:
            A `History` object
         """
        self._load_weights(ckpt)
        history = self.model.fit(
            dataset,
            validation_data=val_dataset,
            callbacks=self.train_callbacks,
            epochs=self.configs['epochs'])
        return history

    def eval(self, dataset, ckpt=None):
        """Evaluate the model.

        Args:
            dataset: Dataset for evaluation, instance of `tf.data.Dataset`
            ckpt: The checkpoint dir or path

        Returns:
            The result of `tf.keras.Model.evaluate(...)`
        """
        self._load_weights(ckpt)
        return self.model.evaluate(dataset, callbacks=self.eval_callbacks)

    def predict(self, dataset, ckpt=None):
        """Predict the model.

        Args:
            dataset: Dataset for prediction
            ckpt: The checkpoint dir or path

        Returns:
            Numpy array of predictions
        """
        self._load_weights(ckpt)
        return self.model.predict(dataset, callbacks=self.predict_callbacks)

    def _init_callbacks(self, train_callbacks=None, eval_callbacks=None, predict_callbacks=None):
        """Extend the callbacks by the default callbacks"""
        default_train_callbacks = [
            tf.keras.callbacks.ModelCheckpoint(
                os.path.join(self.model_dir, '%s-{epoch:04d}.ckpt' % self.model_name),
                save_weights_only=self.configs['ckpt_only_weights'],
                period=self.configs['ckpt_period']),
            tf.keras.callbacks.TensorBoard(log_dir=os.path.join(self.model_dir, 'tensorboard')),
            tf.keras.callbacks.EarlyStopping(patience=self.configs['early_stopping_patience'])
        ]
        if train_callbacks:
            default_train_callbacks.extend(train_callbacks)
        self.train_callbacks = default_train_callbacks

        default_eval_callbacks = []
        if eval_callbacks:
            default_eval_callbacks.extend(eval_callbacks)
        self.eval_callbacks = default_eval_callbacks

        default_predict_callbacks = []
        if predict_callbacks:
            default_predict_callbacks.extend(predict_callbacks)
        self.predict_callbacks = default_predict_callbacks

    def default_config(self):
        """Default configurations. Can be override by `configs` in constructor, or set `self.configs` directly."""
        c = {
            'ckpt_only_weights': True,
            'ckpt_period': 5,
            'early_stopping_patience': 10,
            'epochs': 20,
        }
        return c
